import json
import random
import signal
import httpx
import asyncio

from chouti.util.device import new_app_device

class R_json(dict):
    pass

async def get_link(client: httpx.AsyncClient,linkid: int)->"R_json|None":
    # https://api.chouti.com/links/show.json?id=42479192
    assert isinstance(linkid, int)
    url = f"https://api.chouti.com/links/show.json?id={linkid}"
    response = await client.get(url)
    assert response.status_code == 200

    r_json = response.json()

    if "code" in r_json:
        # "code in response: %s" % r_json
        if r_json["code"] == '400' and r_json["info"] == '找不到该帖子':
            return None
    assert "link" in r_json, "link not in response: %s" % r_json

    return r_json


queue = asyncio.Queue(maxsize=30)

linkid_size_map = []
interrupted = False

async def worker():
    while True:
        client, linkid = await queue.get()
        r_json = await get_link(client,linkid)
        if r_json is None:
            linkid_size_map.append(
            (linkid, 0)
        )
            queue.task_done()
            continue
        print(linkid, len(json.dumps(r_json)))
        queue.task_done()
        linkid_size_map.append(
            (linkid, len(json.dumps(r_json)))
        )

async def interrupt():
    global interrupted
    interrupted = True

async def interrupt_handler():
    global interrupted
    # handle keyboard interrupt, set `interrupted` to True
    try:
        loop = asyncio.get_event_loop()
        for signame in ('SIGINT', 'SIGTERM'):
            loop.add_signal_handler(getattr(signal, signame), lambda: asyncio.create_task(interrupt()))
    except NotImplementedError:
        pass


async def main():
    asyncio.create_task(interrupt_handler())

    async with httpx.AsyncClient(headers={
        "User-Agent": new_app_device(),
    }) as client:

        for _ in range(10):
            asyncio.create_task(worker())

        while linkid := random.randint(0,42479192):
            # r_json = await get_link(client,linkid)
            await queue.put((client,linkid))
            if interrupted:
                break
            # print(linkid, len(json.dumps(r_json)))
        await queue.join()

    # draw the result
    import matplotlib.pyplot as plt
    print("sorting...")
    linkid_size_map.sort(key=lambda x: x[0])
    linkids, sizes = zip(*linkid_size_map)
    # >> result.png
    print("plotting...")
    # x name: linkids
    # y name: sizes
    plt.plot(linkids, sizes)
    plt.savefig("result.png")
    plt.show()
    # avg size
    avg_size = sum(sizes) / len(sizes)
    print("samples:", len(linkids))
    print("avg bytes:", avg_size)

    print("400 rate:", sizes.count(0)/len(sizes))


if __name__ == "__main__":
    asyncio.run(main())